unit EmbeddedFormManagerUnit;

interface

uses
  inifiles,
  extctrls, Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BPLBase, BPLClient;

type
  TEFForm = class(TForm)
    BPLEmbFormMgr: TDynBPLClient;
    procedure BPLEmbFormMgrServiceCall(Sender: TObject;
      ServiceText: String; ServiceNumber: Integer; Subject: TObject;
      out handled: Boolean);
    procedure BPLEmbFormMgrAppObjectNotification(Sender: TObject;
      AppObject: TAppObject; Notification: TAppObjectNotification);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
		ActiveForm: TForm;
		_Closing: boolean;
    IniFileName: string;
    LastFormID: string;
    MainPanel: TPanel;

    procedure AddForm( AppObject: TAppObject );
    procedure RemoveForm( AppObject: TAppObject );
    procedure ChangeForm( Form: TForm );

    function GetAppObjectByForm( Form: TForm ): TAppObject;

    procedure LoadFromIniFile;
    procedure SaveToIniFile;
  public
    { Public declarations }
    procedure OnFormCommandClick( Sender: TObject );
  end;

var
  EFForm: TEFForm;

implementation

{$R *.dfm}


procedure TEFForm.BPLEmbFormMgrServiceCall(Sender: TObject;
  ServiceText: String; ServiceNumber: Integer; Subject: TObject;
  out handled: Boolean);
begin
  SaveToIniFile;
  ChangeForm(nil);
  _Closing := true;
end;

procedure TEFForm.BPLEmbFormMgrAppObjectNotification(Sender: TObject;
  AppObject: TAppObject; Notification: TAppObjectNotification);
begin
  if _Closing
  or (AppObject.ObjectType <> otEmbeddedForm)
  or (AppObject.Component = nil)
  or not (AppObject.Component is TForm) then
    exit;

  case Notification of
    noComponentAssigned,
    noIterate,
    noRegister:
      AddForm(AppObject);
    noUnregister,
    noComponentUnassigning:
      RemoveForm(AppObject);
  end;
end;

procedure TEFForm.FormCreate(Sender: TObject);
var
  appObj: TAppObject;
begin
  _Closing := false;

  appObj :=
    BPLEmbFormMgr.GetNextAppObject(nil, otPath, 'AppConfigFile');
  if appObj <> nil then
    IniFileName := appObj.Text;
  if IniFileName = '' then
    IniFileName := ChangeFileExt( Application.ExeName, '.ini' );

  LoadFromIniFile;
	ActiveForm := nil;
  BPLEmbFormMgr.IterateAppObjects;
end;

procedure TEFForm.AddForm(AppObject: TAppObject);
var
  form: TForm;
  cmdObj: TAppObject;
  bmp: TBitmap;
begin
  if MainPanel = nil then
  begin
    cmdObj := BPLEmbFormMgr.GetNextAppObject(nil,otMainObject,'MainPanel');
    if (cmdObj = nil)
    or (cmdObj.Component = nil)
    or not (cmdObj.Component is TPanel) then
      exit;

    MainPanel := TPanel(cmdObj.Component);
  end;

  // -----------------------------------------

  form := TForm(AppObject.Component);
  if GetAppObjectByForm(form) <> nil then
    exit;

	form.BorderStyle := bsNone;
	form.BorderIcons := [];
	form.TabStop := true;
	form.TabOrder := 0;
	form.Align := alClient;

  // Create Command and register it.

  cmdObj := BPLEmbFormMgr.AppObjects.Add;
  cmdObj.Caption := form.Caption;
  cmdObj.ObjectType := otCommand;
  cmdObj.Data := AppObject.Component;

  bmp := TBitmap.Create;
  try
    if AppObject.Glyph.Width > 0 then
      cmdObj.Glyph := AppObject.Glyph
    else if Assigned(AppObject.ImageList)
    and (AppObject.ImageIndex >= 0) then
    begin
      AppObject.ImageList.GetBitmap(AppObject.ImageIndex,bmp);
      cmdObj.Glyph := bmp;
    end else if not Form.Icon.Empty then
    begin
		  bmp.Width := Form.Icon.Width;
	  	bmp.Height := Form.Icon.Height;
  		bmp.Canvas.Draw( 0, 0, Form.Icon );
      cmdObj.Glyph := bmp;
    end;
  finally
    bmp.Free;
  end;

  cmdObj.Enabled := true;
  cmdObj.Visible := true;
  cmdObj.Checked := false;
  cmdObj.Category := AppObject.Category;
  cmdObj.MenuPath := AppObject.MenuPath;
  cmdObj.Text := AppObject.ID;
  if cmdObj.Category = '' then
    cmdObj.Category := AppObject.BPLObject.Caption;

  cmdObj.OnExecute := OnFormCommandClick;
  cmdObj.RegisterObject;

  // ------------------------------------------------
  // We load the first form or - if we saved a form
  // in a previous session, the last used one.
  // ------------------------------------------------

  if (LastFormID <> '')
  and (AppObject.ID = LastFormID) then
  begin
    LastFormID := '';
    ChangeForm(TForm(AppObject.Component));
  end else
    if ActiveForm = nil then
      ChangeForm(TForm(AppObject.Component));

end;

procedure TEFForm.RemoveForm(AppObject: TAppObject);
var
  cmdObj, hlpObj: TAppObject;
  form, newForm: TForm;
  newIndex: integer;
begin
  form := TForm(AppObject.Component);
  cmdObj := GetAppObjectByForm(form);
  if cmdObj = nil then
    exit;

  form.Visible := false;
  newIndex := -1;
  newForm := nil;

  if form = ActiveForm then
  begin
    if cmdObj.Index > 0 then
      newIndex := cmdObj.Index-1
    else if cmdObj.Index < BPLEmbFormMgr.AppObjects.Count-1 then
      newIndex := cmdObj.Index+1;

    if newIndex < 0 then
      if BPLEmbFormMgr.AppObjects.Count > 0 then
        newIndex := 0;

    if newIndex >= 0 then
    begin
      hlpObj := BPLEmbFormMgr.AppObjects.Items[newIndex];
      newForm := TForm(hlpObj.Component);
    end;

    ChangeForm(newForm);
  end;

  BPLEmbFormMgr.AppObjects.Delete(cmdObj.Index);
end;

procedure TEFForm.OnFormCommandClick(Sender: TObject);
begin
  ChangeForm( TForm(TAppObject(Sender).Data) );
end;

procedure TEFForm.ChangeForm(Form: TForm);
var
  cmdObj: TAppObject;
begin
  if ActiveForm <> nil then
  begin
    ActiveForm.Visible := false;
    ActiveForm.Parent := nil;
    cmdObj := GetAppObjectByForm(ActiveForm);
    if cmdObj <> nil then
      cmdObj.Checked := false;
  end;
  ActiveForm := Form;
  if ActiveForm <> nil then
  begin
    ActiveForm.Parent := MainPanel;
    ActiveForm.Visible := true;
    cmdObj := GetAppObjectByForm(Form);
    if cmdObj <> nil then
      cmdObj.Checked := true;
  end;
end;

function TEFForm.GetAppObjectByForm(Form: TForm): TAppObject;
var
  i: integer;
begin
  Result := nil;
  for i:=0 to BPLEmbFormMgr.AppObjects.Count-1 do
  begin
    if BPLEmbFormMgr.AppObjects.Items[i].Data = Form then
    begin
      Result := BPLEmbFormMgr.AppObjects.Items[i];
      exit;
    end;
  end;
end;

procedure TEFForm.FormDestroy(Sender: TObject);
begin
  ChangeForm(nil);
end;

procedure TEFForm.LoadFromIniFile;
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(IniFileName);
  try
    LastFormID :=
      ini.ReadString('EmbeddedFormManager','LastFormID','');
  finally
    ini.Free;
  end;
end;

procedure TEFForm.SaveToIniFile;
var
  obj: TAppObject;
  ini: TIniFile;
begin
  ini := TIniFile.Create(IniFileName);
  try
    obj := GetAppObjectByForm(ActiveForm);
    if obj = nil then
      LastFormID := ''
    else
      LastFormID := obj.Text;
      
    ini.WriteString('EmbeddedFormManager','LastFormID',LastFormID);
  finally
    ini.Free;
  end;
end;

initialization
  EFForm := TEFForm.Create(nil);

finalization
  EFForm.Free;

end.
